"""
Command line interface for SoupStars
"""


import click
import json
import os
import sys
import textwrap
import getpass
from pygments import highlight, lexers, formatters

from soupstars.core import SoupStar
from soupstars.client import Client
from soupstars import __version__


pygments_style = "native"


def jsonify(data):
    formatted_json = json.dumps(data, sort_keys=True, indent=2)
    colorful_json = highlight(
        formatted_json,
        lexers.JsonLexer(),
        formatters.Terminal256Formatter(style=pygments_style)
    )
    print(colorful_json)


@click.group()
def cli():
    """
    CLI to interact with SoupStars cloud.
    """

    pass


@cli.command()
def login():
    """
    Log in with an existing email
    """

    client = Client()
    email = input('Email: ')
    password = getpass.getpass(prompt='Password: ')

    resp = client.login(email=email, password=password)

    if resp.ok:
        client.token = resp.json()['token']
        client.update_config()

    jsonify(resp.json())


@cli.command()
def ls():
    """
    Show the parsers uploaded to SoupStars cloud
    """

    client = Client()
    resp = client.ls()
    jsonify(resp.json())


@cli.command()
def health():
    """
    Print the status of the SoupStars api
    """

    resp = Client().health()
    jsonify(resp.json())


@cli.command()
def register():
    """
    Register a new account on SoupStars cloud
    """

    email = input('Email: ')
    password = getpass.getpass(prompt='Password: ')
    password2 = getpass.getpass(prompt='Confirm password: ')

    if password != password2:
        print("Passwords did not match.")
        return

    client = Client()
    resp = client.register(email=email, password=password)

    if resp.ok:
        client.token = resp.json()['token']
        client.update_config()

    jsonify(resp.json())


@cli.command()
def whoami():
    """
    Print the email address of the current user
    """

    client = Client()
    resp = client.profile()
    jsonify(resp.json())


@cli.command()
def config():
    """
    Print the configuration used by the client
    """

    client = Client()
    data = {
        "soupstars": {
            "version": __version__,
        },
        "python": {
            "version": f"{sys.version})".replace('\n', ''),
            "path": sys.executable,
        },
        "platform": sys.platform,
        "config": client.config(),
    }

    jsonify(data)


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to debug")
def debug(module):
    """
    Open a python prompt with a parser result
    """

    if module[-3:] == ".py":
        module = module[:-3]

    if '.' not in sys.path:
        sys.path.append('.')

    soupstar = SoupStar(module)
    soupstar.load()

    response = soupstar.response  # noqa
    request = soupstar.response.request  # noqa
    url = soupstar.url  # noqa
    soup = soupstar.soup  # noqa

    try:
        import IPython
        IPython.embed(colors="neutral")
    except ImportError:
        import code
        code.InteractiveConsole(locals=vars()).interact()


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to create")
@click.option('--force/--no-force', default=False,
              help="Overwrite an existing file")
def create(module, force):
    """
    Create a new parser from a template
    """

    if module[-3:] != '.py':
        module = module + ".py"

    if os.path.exists(module) and not force:
        raise FileExistsError(
            f"{module} already exists. Use --force to overwrite."
        )

    template = textwrap.dedent("""\
    from soupstars import parse, SoupStar

    url = "https://corbettanalytics.com/"

    @parse
    def h1(soup):
        return soup.h1.text

    if __name__ == "__main__":
        print(SoupStar(__name__).to_dict())

    """)

    with open(module, 'w') as o:
        o.write(template)
        jsonify({"state": "done", "module": module})


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to push")
def push(module):
    """
    Push a parser to SoupStars cloud
    """

    client = Client()
    resp = client.push(module)
    jsonify(resp.json())


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to pull")
def pull(module):
    """
    Pull a parser from SoupStars cloud into a local module
    """

    client = Client()
    resp = client.pull(module)
    data = resp.json()
    with open(data['name'], 'w') as o:
        o.write(data['module'])
    jsonify({"state": "done", "response": data})


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to create")
def run(module):
    """
    Run a parser on SoupStars cloud
    """

    client = Client()
    resp = client.run(module)
    jsonify(resp.json())


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to show")
@click.option('--json/--no-json', default=False,
              help="Show parser details in JSON")
def show(module, json):
    """
    Show the contents of a parser on SoupStars cloud
    """

    client = Client()
    resp = client.pull(module)

    if json:
        jsonify(resp.json())
    else:
        module = highlight(
            resp.json()['module'],
            lexers.PythonLexer(),
            formatters.Terminal256Formatter(style=pygments_style)
        )
        print(module)


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to test")
def test(module):
    """
    Test running a parser locally
    """

    if module[-3:] == ".py":
        module = module[:-3]

    if '.' not in sys.path:
        sys.path.append('.')

    soupstar = SoupStar(module)
    jsonify(soupstar.to_dict())


@cli.command()
@click.option('--module', '-m', required=True,
              help="Name of the parser to test")
def results(module):
    """
    Print results of a parser
    """

    client = Client()
    resp = client.results(module)
    jsonify(resp.json())



import os
import requests
import json


class Client(object):
    """
    Wrapper for the SoupStars web API.

    >>> from soupstars import Client
    >>> client = Client()
    >>> resp = client.status()
    >>> resp.json['healthy']
    True
    """

    default_config = {
        'token': None,
        'host': "https://soupstars-cloud.herokuapp.com"
    }

    def __init__(self, config_dir=None, config_file=None, host=None,
                 token=None):
        self.config_file = config_file or "config.json"
        self.config_dir = config_dir or os.environ.get('SOUPSTARS_HOME') \
            or os.path.join(os.path.expanduser('~'), '.soupstars')
        self.config_path = os.path.join(self.config_dir, self.config_file)
        config = self._load_config()
        self.host = host or config['host']
        self.token = token or config['token']

    def _write_config(self, config=default_config):
        if not os.path.exists(self.config_dir):
            os.mkdir(self.config_dir)
        with open(self.config_path, 'w') as o:
            json.dump(config, o, indent=2)

    def _load_config(self):
        if not os.path.exists(self.config_path):
            self._write_config()
            return self._load_config()
        else:
            with open(self.config_path) as o:
                return json.load(o)

    def update_config(self):
        config = self.config()
        config.pop('path')
        self._write_config(config=config)

    def config(self):
        return {
            'host': self.host,
            'token': self.token,
            'path': self.config_path
        }

    def send(self, path, method='POST', data=None, files=None):
        url = self.host + path
        headers = {'Authorization': self.token}
        resp = requests.request(
            url=url,
            method=method,
            json=data,
            headers=headers,
            files=files
        )
        return resp

    def register(self, email, password):
        data = {
            'email': email,
            'password': password
        }
        return self.send('/register', data=data)

    def login(self, email, password):
        data = {
            'email': email,
            'password': password
        }
        return self.send('/token', data=data)

    def health(self):
        return self.send('/health', method='GET')

    def ls(self):
        return self.send('/list')

    def profile(self):
        return self.send('/profile')

    def push(self, module):
        file_data = open(module, 'rb').read()
        files = {module: file_data}
        return self.send('/push', files=files)

    def pull(self, module):
        data = {'name': module}
        return self.send('/pull', data=data)

    def run(self, module):
        data = {'name': module}
        return self.send('/run', data=data)

    def results(self, module):
        data = {'name': module}
        return self.send('/results', data=data)
